from typing import Dict, Optional, List

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.models import MediaType


class Person:
    id: int
    douban_id: int
    imdb_id: str
    cn_name: str
    en_name: str
    aka_cn_names: str
    aka_en_names: str
    country: str
    date_of_birth: str
    douban_image_url: str
    description: str
    role: str
    duty: str
    order_number: int

    def __init__(self, data: Dict):
        utils.copy_value(data, self, True)


class MetaApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get_cast_crew_by_douban(self, media_type: MediaType, douban_id: int):
        list_ = self._session.get('meta.get_cast_crew_by_douban', {
            'media_type': media_type.value,
            'douban_id': douban_id
        })
        if not list_:
            return
        return [Person(x) for x in list_]

    def get_cast_crew_by_tmdb(self, media_type: MediaType, tmdb_id: int, season_number: Optional[int] = None) -> \
            Optional[List[Person]]:
        list_ = self._session.get('meta.get_cast_crew_by_tmdb', {
            'media_type': media_type.value,
            'tmdb_id': tmdb_id,
            'season_number': season_number
        })
        if not list_:
            return
        return [Person(x) for x in list_]



from typing import List, Dict, Optional

from moviebotapi import Session
from moviebotapi.core import utils


class MediaNameMeta:
    cn_name: str = None
    aka_names: List[str] = []
    en_name: str = None
    year: int = None
    season_number: List[int] = None
    episode_number: List[int] = None
    resolution: str = None
    media_source: str = None
    media_codec: str = None
    media_audio: List[str] = None
    release_team: str = None

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class AmrApi:
    """
    Automatic media Recognition
    自动媒体信息分析接口
    """

    def __init__(self, session: Session):
        self._session: Session = session

    def parse_name_meta_by_string(self, string: str) -> Optional[MediaNameMeta]:
        res = self._session.get('amr.parse_name_meta_by_string', {
            'string': string
        })
        if not res:
            return
        return MediaNameMeta(res)

    def parse_name_meta_by_filepath(self, filepath: str) -> Optional[MediaNameMeta]:
        res = self._session.get('amr.parse_name_meta_by_filepath', {
            'filepath': filepath
        })
        if not res:
            return
        return MediaNameMeta(res)



from enum import Enum
from typing import Dict, Optional

from moviebotapi import Session


class SystemNotifyType(str, Enum):
    Login = "登陆"
    SmartDownload = "智能下载"
    System = "系统消息"
    SystemError = "系统错误"
    Plugin = "插件消息"


class NotifyApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def send_message_by_tmpl_name(self, template_name: str, context: Dict, to_uid: Optional[int] = None):
        self._session.post('notify.send_message_by_tmpl_name', {
            'to_uid': to_uid,
            'template_name': template_name,
            'context': context
        })

    def send_message_by_tmpl(self, title: str, body: str, context: Dict, to_uid: Optional[int] = None):
        self._session.post('notify.send_message_by_tmpl_name', {
            'to_uid': to_uid,
            'title': title,
            'body': body,
            'context': context
        })

    def send_text_message(self, title: str, body: str, to_uid: Optional[int] = None):
        self._session.post('notify.send_text_message', {
            'to_uid': to_uid,
            'title': title,
            'body': body
        })

    def send_system_message(self, to_uid: Optional[int], title: str, message: str,
                            message_type: SystemNotifyType = None):
        if not message_type:
            message_type = SystemNotifyType.Plugin
        self._session.post('notify.send_system_message', {
            'to_uid': to_uid,
            'title': title,
            'message': message,
            'type': message_type.name
        })

    def clear_system_message(self, uid: int):
        self._session.get('notify.clear_system_message', {
            'uid': uid
        })



import datetime
from enum import Enum
from typing import Dict, Optional, List, Union

from moviebotapi import Session
from moviebotapi.core import utils


class CateLevel1(str, Enum):
    Movie = 'Movie'
    TV = 'TV'
    Documentary = 'Documentary'
    Anime = 'Anime'
    Music = 'Music'
    Game = 'Game'
    AV = 'AV'
    Other = 'Other'

    @staticmethod
    def get_type(enum_name: str):
        for item in CateLevel1:
            if item.name == enum_name:
                return item
        return


class Torrent:
    # 种子id
    id: str
    # 站点编号
    site_id: str
    gmt_modified: datetime.datetime
    # 种子编号
    # torrent_id: int = db.Column(db.Integer, nullable=False, primary_key=True, unique=True)
    # 种子名称
    name: str
    # 种子标题
    subject: str
    # 以及类目
    cate_level1: CateLevel1
    # 站点类目id
    cate_id: str
    # 种子详情页地址
    details_url: str
    # 种子下载链接
    download_url: str
    # 种子关联的imdbid
    imdb_id: str
    # 种子发布时间
    publish_date: datetime.datetime
    # 种子大小，转化为mb尺寸
    size_mb: float
    # 做种人数
    upload_count: int
    # 下载中人数
    downloading_count: int
    # 下载完成人数
    download_count: int
    # 免费截止时间
    free_deadline: datetime.datetime
    # 下载折扣，1为不免费
    download_volume_factor: float
    # 做种上传系数，1为正常
    upload_volume_factor: int
    minimum_ratio: float
    minimum_seed_time: int
    # 封面链接
    poster_url: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class SearchType(str, Enum):
    Keyword = 'keyword'
    Imdb = 'imdb_id'


class SearchQuery:
    key: SearchType
    value: str

    def __init__(self, key: SearchType, value: str):
        self.key = key
        self.value = value


class TrafficManagementStatus(int, Enum):
    Disabled = 0
    Initiative = 1
    Passive = 2


class SiteStatus(int, Enum):
    Normal = 1
    Error = 2


class Site:
    id: int
    gmt_modified: datetime.datetime
    uid: int
    username: str
    cookie: str
    web_search: bool
    smart_download: bool
    share_rate: float
    upload_size: float
    download_size: float
    is_vip: bool
    status: SiteStatus
    traffic_management_status: TrafficManagementStatus
    # 主动模式时上传流量目标，单位GB
    upload_kpi: int
    proxies: str
    user_agent: str

    def __init__(self, data: Dict, api: "SiteApi"):
        utils.copy_value(data, self)
        self.site_id: str = utils.parse_value(str, data.get('site_name'))
        self.site_name: str = utils.parse_value(str, data.get('alias'))
        self._api = api

    def update(self):
        self._api.update(self.site_id, self.cookie, self.web_search, self.smart_download,
                         self.traffic_management_status, self.upload_kpi, self.proxies, self.user_agent)

    def delete(self):
        self._api.delete(self.id)


class SiteApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def list(self):
        list_ = self._session.get('site.get_sites')
        if not list_:
            return
        return [Site(x, self) for x in list_]

    def update(self, site_id: str, cookie: str,
               web_search: Optional[bool] = None, smart_download: bool = None,
               traffic_management_status: Optional[TrafficManagementStatus] = None,
               upload_kpi: Optional[int] = None, proxies: Optional[str] = None,
               user_agent: Optional[str] = None):
        self._session.post('site.save_site', json={
            'site_name': site_id,
            'cookie': cookie,
            'web_search': web_search,
            'smart_download': smart_download,
            'traffic_management_status': traffic_management_status,
            'upload_kpi': upload_kpi,
            'proxies': proxies,
            'user_agent': user_agent
        })

    def delete(self, id_: int):
        self._session.post('site.delete', {
            'id': id_
        })

    def search_local(self, query: Union[SearchQuery, List[SearchQuery]],
                     cate_level1: Optional[List[CateLevel1]] = None) -> Optional[List[Torrent]]:
        if isinstance(query, SearchQuery):
            query = [{'key': query.key.value, 'value': query.value}]
        elif isinstance(query, list):
            query = [{'key': str(x.key), 'value': x.value} for x in query]
        torrents = self._session.post('site.search_local', {
            'query': query,
            'cate_level1': cate_level1
        })
        if not torrents:
            return []
        return [Torrent(x) for x in torrents]



from typing import Dict

from moviebotapi import Session
from moviebotapi.core import utils


class DoubanConfig:
    cookie: str

    def __init__(self, data: Dict, session: Session):
        utils.copy_value(data, self)
        self._ = session

    def save(self):
        """
        保存豆瓣配置，保存过程服务端会检查cookie的有效性，不是简单的保存配置文件
        """
        self._.post('setting.save_douban', json={
            'cookie': self.cookie
        })


class ConfigApi:
    def __init__(self, session: Session):
        self._session: Session = session

    @property
    def douban(self):
        return DoubanConfig(self._session.get('setting.get_douban'), self._session)



from enum import Enum


class MediaType(str, Enum):
    Movie = 'Movie'
    TV = 'TV'

    @staticmethod
    def get(value):
        l = str(value).lower()
        if l == 'movie':
            return MediaType.Movie
        if l in ['tv', 'series']:
            return MediaType.TV



from typing import Optional, List, Dict, Union

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.decorators import ignore_attr_not_exists
from moviebotapi.core.models import MediaType


class TmdbGenres:
    id: int
    name: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class ProductionCompanies:
    id: int
    logo_path: str
    name: str
    origin_country: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class SpokenLanguages:
    english_name: str
    iso_639_1: str
    name: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class SearchResultItem:
    adult: bool
    backdrop_path: str
    genre_ids: List[int]
    id: int
    media_type: MediaType
    original_language: str
    original_title: str
    overview: str
    popularity: float
    poster_path: str
    release_date: str
    title: str
    video: bool
    vote_average: float
    vote_count: int

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class SearchResult:
    page: int
    total_pages: int
    total_results: int
    results: List[Union[SearchResultItem, "TmdbPerson"]]

    def __init__(self, data: Dict):
        self.page = utils.parse_value(int, data.get('page'))
        self.total_pages = utils.parse_value(int, data.get('total_pages'))
        self.total_results = utils.parse_value(int, data.get('total_results'))
        if data.get('results'):
            res = []
            for item in data.get('results'):
                if item.get('media_type') == 'person':
                    res.append(TmdbPerson(item))
                else:
                    res.append(SearchResultItem(item))
            self.results = res
        else:
            self.results = []


@ignore_attr_not_exists
class TmdbMovie:
    id: int
    imdb_id: str
    adult: bool
    backdrop_path: str
    belongs_to_collection: str
    budget: int
    genres: List[TmdbGenres]
    homepage: str
    original_language: str
    title: str
    original_title: str
    overview: str
    popularity: float
    poster_path: str
    release_date: str
    revenue: int
    runtime: int
    status: str
    tagline: str
    video: bool
    vote_average: float
    vote_count: int
    production_companies: List[ProductionCompanies]
    production_countries: List
    spoken_languages: List[SpokenLanguages]

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class EpisodeMeta:
    air_date: str
    episode_number: int
    id: int
    name: str
    overview: str
    production_code: str
    runtime: int
    season_number: int
    show_id: int
    still_path: str
    vote_average: float
    vote_count: int

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class Network:
    id: int
    name: str
    logo_path: str
    origin_country: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class Season:
    air_date: str
    episode_count: int
    id: int
    name: str
    overview: str
    poster_path: str
    season_number: int

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


@ignore_attr_not_exists
class TmdbTV:
    name: str
    original_name: str
    id: int
    in_production: bool
    adult: bool
    backdrop_path: str
    first_air_date: str
    last_air_date: str
    episode_run_time: List[int]
    genres: List[TmdbGenres]
    homepage: str
    languages: List[str]
    number_of_episodes: int
    number_of_seasons: int
    origin_country: List[str]
    original_language: List[str]
    overview: str
    popularity: float
    poster_path: str
    production_companies: List[ProductionCompanies]
    production_countries: List
    spoken_languages: List[SpokenLanguages]
    status: str
    tagline: str
    type: str
    vote_average: float
    vote_count: int
    created_by: dict
    last_episode_to_air: EpisodeMeta
    next_episode_to_air: EpisodeMeta
    networks: List[Network]
    seasons: List[Season]

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class TmdbPerson:
    adult: bool
    gender: int
    id: int
    known_for: List[SearchResultItem]
    known_for_department: str
    name: str
    popularity: float
    profile_path: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class TmdbApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get(self, media_type: MediaType, tmdb_id: int, language: Optional[str] = None) -> Union[
        TmdbMovie, TmdbTV, None]:
        if not language:
            language = 'zh-CN'
        meta = self._session.get('tmdb.get', {
            'media_type': media_type.value,
            'tmdb_id': tmdb_id,
            'language': language
        })
        if not meta:
            return
        if media_type == MediaType.Movie:
            return TmdbMovie(meta)
        else:
            return TmdbTV(meta)

    def search(self, media_type: MediaType, query: str, year: Optional[int] = None, language: Optional[str] = None):
        res = self._session.get('tmdb.search', {
            'media_type': media_type.value,
            'query': query,
            'language': language,
            'year': year
        })
        if not res:
            return
        result = []
        for item in res:
            item['media_type'] = media_type.value
            result.append(SearchResultItem(item))
        return result

    def search_multi(self, query: str, language: Optional[str] = None, page: Optional[int] = None) -> Optional[
        SearchResult]:
        res = self._session.get('tmdb.search_multi', {
            'query': query,
            'language': language,
            'page': page
        })
        if not res:
            return
        return SearchResult(res)
